// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AISS8AO4/16aiss8ao4_Linux_3.x.x.x_gsc_dn/utils/util_fsamp_ai.c $
// $Rev: 54006 $
// $Date: 2023-12-12 08:58:57 -0600 (Tue, 12 Dec 2023) $

// 16AISS8AO4: Utilities: source file

#include "main.h"



// macros *********************************************************************

#define	ABS(v)					(((v) < 0) ? (-(v)) : (v))
#define	LIMIT_MAX(v,max)		(((v) > (max)) ? (max) : (v))
#define	LIMIT_MIN(v,min)		(((v) < (min)) ? (min) : (v))
#define	LIMIT_RANGE(v,min,max)	(LIMIT_MIN((LIMIT_MAX((v),(max))),(min)))



/******************************************************************************
*
*	Function:	aiss8ao4_fsamp_ai_compute
*
*	Purpose:
*
*		Determine the best configuration for the specified output sample rate.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		fsamp	The desired sample rate.
*
*		ndiv	The Ndiv value is put here.
*
*		rate	Report the sample rate achieved here, if non-NULL.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int aiss8ao4_fsamp_ai_compute(int fd, int index, int verbose, s32 fsamp, s32* ndiv, double* rate)
{
	double	delta;
	double	delta_best	= FLT_MAX;
	int		errs		= 0;
	double	fsamp_best	= 0;
	double	fsamp_got;
	s32		fsamp_max;	// Queried value.
	s32		fsamp_min;	// Queried value.
	double	fsamp_want;
	int		init		= 0;	// Have we performed the first run initialization?
	s32		master;
	s32		nd;
	s32		nd_max;
	s32		nd_min;
	s32		nd_use;
	s32		ndiv_best	= 0;
	s32		ndiv_max;	// Queried value.
	s32		ndiv_min;	// Queried value.

	if (verbose)
	{
		gsc_label_index("AI Sample Rate", index);
		gsc_label_long_comma(fsamp);
		printf(" S/S");
	}

	errs	+= aiss8ao4_query(fd, -1, 0, AISS8AO4_QUERY_FSAMP_MAX_AI, &fsamp_max);
	errs	+= aiss8ao4_query(fd, -1, 0, AISS8AO4_QUERY_FSAMP_MIN_AI, &fsamp_min);
	errs	+= aiss8ao4_query(fd, -1, 0, AISS8AO4_QUERY_MASTER_CLOCK, &master);
	errs	+= aiss8ao4_query(fd, -1, 0, AISS8AO4_QUERY_NDIV_MAX_AI, &ndiv_max);
	errs	+= aiss8ao4_query(fd, -1, 0, AISS8AO4_QUERY_NDIV_MIN_AI, &ndiv_min);

	fsamp_want	= (double) LIMIT_RANGE(fsamp, fsamp_min, fsamp_max);
	nd			= master / fsamp_want;
	nd_min		= nd - 1;
	nd_max		= nd + 1;
	nd_min		= (nd_min < ndiv_min) ? ndiv_min : nd_min;
	nd_max		= (nd_max > ndiv_max) ? ndiv_max : nd_max;

	for (nd = nd_min; nd <= nd_max; nd++)
	{
		if (nd < nd_min)
			nd_use	= nd_min;
		else if (nd > nd_max)
			nd_use	= nd_max;
		else
			nd_use	= nd;

		fsamp_got	= ((double) master) / nd_use;
		delta		= fsamp_got - fsamp_want;

		if ((init == 0) || ((ABS(delta)) < (ABS(delta_best))))
		{
			init		= 1;
			delta_best	= delta;
			fsamp_best	= fsamp_got;
			ndiv_best	= nd_use;
		}
	}

	ndiv[0]		= ndiv_best;

	if (rate)
		rate[0]	= fsamp_best;

	if (verbose)
	{
		printf("%s  (", errs ? " FAIL <---" : "");
		printf("Fref ");
		gsc_label_long_comma((long) master);
		printf(", Ndiv ");
		gsc_label_long_comma((long) ndiv_best);
		printf(", Fsamp %.3f", fsamp_best);
		printf(")\n");
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	aiss8ao4_fsamp_ai_report
*
*	Purpose:
*
*		Determine and report the input sample rate for the specified channel.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	0: errors only, !0: all output
*
*		chan	The index of the channel of interest.
*
*		fsamp	Report the sample rate here, if non-NULL.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int aiss8ao4_fsamp_ai_report(int fd, int index, int verbose, int chan, s32* fsamp)
{
	s32		bit			= 0x1L << chan;
	char	buf[64];
	s32		burst;
	s32		clk_src;
	s32		enable;
	int		errs		= 0;
	s32		gen_c;
	s32		master;
	s32		ndiv;
	s32		sel;
	s32		set			= 0;

	if (verbose)
	{
		sprintf(buf, "AI Channel %d", chan);
		gsc_label_index(buf, index);
	}

	errs	+= aiss8ao4_query(fd, -1, 0, AISS8AO4_QUERY_MASTER_CLOCK, &master);
	errs	+= aiss8ao4_ai_chan_sel		(fd, -1, 0, -1, &sel);
	errs	+= aiss8ao4_ai_burst_enable	(fd, -1, 0, -1, &burst);
	errs	+= aiss8ao4_gen_a_enable	(fd, -1, 0, -1, &gen_c);
	errs	+= aiss8ao4_ai_clock_src	(fd, -1, 0, -1, &clk_src);
	errs	+= aiss8ao4_ai_enable		(fd, -1, 0, -1,	&enable);
	errs	+= aiss8ao4_gen_a_ndiv		(fd, -1, 0, -1, &ndiv);

	if (errs)
	{
	}
	else if ((sel & bit) == 0)
	{
		if (verbose)
			printf("0 S/S  (Channel Disabled)\n");
	}
	else if (burst == AISS8AO4_BURST_ENABLE_YES)
	{
		if (verbose)
			printf("0 S/S  (Bursting Enabled)\n");
	}
	else if (clk_src == AISS8AO4_AI_CLOCK_SRC_EXT)
	{
		if (verbose)
			printf("0 S/S  (Clock Source is External)\n");
	}
	else if (gen_c == AISS8AO4_GEN_ENABLE_NO)
	{
		if (verbose)
			printf("0 S/S  (Rate-A Generator is disabled)\n");
	}
	else if (enable == AISS8AO4_AI_ENABLE_NO)
	{
		if (verbose)
			printf("0 S/S  (AI is disabled)\n");
	}
	else
	{
		set	= master / ndiv;

		if (verbose)
		{
			gsc_label_long_comma(set);
			printf(" S/S  (Master ");
			gsc_label_long_comma(master);
			printf(" Hz, Ndiv ");
			gsc_label_long_comma(ndiv);
			printf(")\n");
		}
	}

	if (fsamp)
		fsamp[0]	= set;

	return(errs);
}



/******************************************************************************
*
*	Function:	aiss8ao4_fsamp_ai_report_all
*
*	Purpose:
*
*		Determine and report the input sample rate for all channels.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	0: errors only, !0: all output
*
*		fsamp	Report the sample rate here, if non-NULL.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int aiss8ao4_fsamp_ai_report_all(int fd, int index, int verbose, s32* fsamp)
{
	s32	chans;
	int	chan;
	int	errs;
	s32	sps;
	s32	total	= 0;

	if (verbose)
		gsc_label_index("AI Sample Rates", index);

	errs	= aiss8ao4_query(fd, -1, 0, AISS8AO4_QUERY_CHANNEL_AI_QTY, &chans);

	if (errs == 0)
	{
		if (verbose)
			printf("\n");

		gsc_label_level_inc();

		for (chan = 0; chan < chans; chan++)
		{
			fsamp	= 0;
			errs	+= aiss8ao4_fsamp_ai_report(fd, -1, verbose, chan, &sps);
			total	+= sps;
		}

		if (verbose)
		{
			gsc_label("Overall AI Rate");
			gsc_label_long_comma(total);
			printf(" S/S\n");
		}

		gsc_label_level_dec();
	}

	if (fsamp)
		fsamp[0]	= total;

	return(errs);
}


